@@ -1359,12 +1359,15 @@ TEST_F(ClassifyTest, dropClass) {
     // Option, dropped.
     EXPECT_FALSE(client2.getContext().response_);
 
-    // There should also be pkt4-receive-drop stat bumped up.
+    // There should also be stats bumped up.
     stats::StatsMgr& mgr = stats::StatsMgr::instance();
     stats::ObservationPtr drop_stat = mgr.getObservation("pkt4-receive-drop");
+    stats::ObservationPtr filt_stat = mgr.getObservation("pkt4-admin-filtered");
 
-    // This statistic must be present and must be set to 1.
+    // These statistics must be present and must be set to 1.
+    ASSERT_TRUE(filt_stat);
     ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, filt_stat->getInteger().first);
     EXPECT_EQ(1, drop_stat->getInteger().first);
 }
 
@@ -1395,12 +1398,15 @@ TEST_F(ClassifyTest, dropClassUnknown) {
     // No reservation, dropped.
     EXPECT_FALSE(client2.getContext().response_);
 
-    // There should also be pkt4-receive-drop stat bumped up.
+    // There should also be stats bumped up.
     stats::StatsMgr& mgr = stats::StatsMgr::instance();
+    stats::ObservationPtr filt_stat = mgr.getObservation("pkt4-admin-filtered");
     stats::ObservationPtr drop_stat = mgr.getObservation("pkt4-receive-drop");
 
-    // This statistic must be present and must be set to 1.
+    // These statistics must be present and must be set to 1.
+    ASSERT_TRUE(filt_stat);
     ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, filt_stat->getInteger().first);
     EXPECT_EQ(1, drop_stat->getInteger().first);
 }
 
@@ -1431,12 +1437,15 @@ TEST_F(ClassifyTest, dropClassReservedClass) {
     // No reservation, dropped.
     EXPECT_FALSE(client2.getContext().response_);
 
-    // There should also be pkt4-receive-drop stat bumped up.
+    // There should also be stats bumped up.
     stats::StatsMgr& mgr = stats::StatsMgr::instance();
+    stats::ObservationPtr filt_stat = mgr.getObservation("pkt4-admin-filtered");
     stats::ObservationPtr drop_stat = mgr.getObservation("pkt4-receive-drop");
 
-    // This statistic must be present and must be set to 1.
+    // These statistics must be present and must be set to 1.
+    ASSERT_TRUE(filt_stat);
     ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, filt_stat->getInteger().first);
     EXPECT_EQ(1, drop_stat->getInteger().first);
 }
 
@@ -1490,12 +1499,15 @@ TEST_F(ClassifyTest, earlyDrop) {
     // Match the reservation so dropped.
     EXPECT_FALSE(client.getContext().response_);
 
-    // There should also be pkt4-receive-drop stat bumped up.
+    // There should also be stats bumped up.
     stats::StatsMgr& mgr = stats::StatsMgr::instance();
+    stats::ObservationPtr filt_stat = mgr.getObservation("pkt4-admin-filtered");
     stats::ObservationPtr drop_stat = mgr.getObservation("pkt4-receive-drop");
 
-    // This statistic must be present and must be set to 1.
+    // These statistics must be present and must be set to 1.
+    ASSERT_TRUE(filt_stat);
     ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, filt_stat->getInteger().first);
     EXPECT_EQ(1, drop_stat->getInteger().first);
 
     // Try with a different HW address.
