@@ -11,7 +11,9 @@
 #include <host_cache_parsers.h>
 #include <host_cache_log.h>
 #include <database/db_exceptions.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <util/encode/encode.h>
+#include <util/filesystem.h>
 #include <util/multi_threading_mgr.h>
 #include <util/str.h>
 #include <string>
@@ -25,6 +27,7 @@ using namespace isc::data;
 using namespace isc::db;
 using namespace isc::dhcp;
 using namespace isc::util;
+using namespace isc::util::file;
 
 namespace isc {
 namespace host_cache {
@@ -750,9 +753,14 @@ HostCache::cacheWriteHandler(hooks::CalloutHandle& handle) {
             isc_throw(BadValue, "invalid (not a string) parameter");
         }
 
-        filename = cmd_args_->stringValue();
-        if (filename.empty()) {
-            isc_throw(BadValue, "invalid (empty string) parameter");
+        try {
+            filename = CfgMgr::instance().validatePath(cmd_args_->stringValue());
+        } catch (const SecurityWarn& ex) {
+            LOG_WARN(host_cache_logger, HOST_CACHE_PATH_SECURITY_WARNING)
+                    .arg(ex.what());
+            filename = cmd_args_->stringValue();
+        } catch (const std::exception& ex) {
+            isc_throw(BadValue, "parameter is invalid: " << ex.what());
         }
 
         ofstream out(filename, ios::trunc);
