@@ -442,6 +442,7 @@ public:
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 99, "declined-addresses"),
                         int64_t(2));
+                    isc::stats::StatsMgr::instance().setValue("declined-addresses", int64_t(4));
                 } else {
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 66, "declined-addresses"),
@@ -449,6 +450,7 @@ public:
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 99, "declined-addresses"),
                         int64_t(0));
+                    isc::stats::StatsMgr::instance().setValue("declined-addresses", int64_t(0));
                 }
                 isc::stats::StatsMgr::instance().setValue(
                         isc::stats::StatsMgr::generateName("subnet", 66, "assigned-nas" ),
@@ -456,6 +458,7 @@ public:
                 isc::stats::StatsMgr::instance().setValue(
                         isc::stats::StatsMgr::generateName("subnet", 99, "assigned-nas" ),
                     int64_t(2));
+                isc::stats::StatsMgr::instance().setValue("assigned-nas", int64_t(4));
             } else {
                 lmptr_->addLease(createLease4("192.0.2.1", 44, 0x08, 0x42, declined));
                 lmptr_->addLease(createLease4("192.0.2.2", 44, 0x09, 0x56, declined, 5));
@@ -468,6 +471,7 @@ public:
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 88, "declined-addresses"),
                         int64_t(2));
+                    isc::stats::StatsMgr::instance().setValue("declined-addresses", int64_t(4));
                 } else {
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 44, "declined-addresses"),
@@ -475,6 +479,7 @@ public:
                     isc::stats::StatsMgr::instance().setValue(
                             isc::stats::StatsMgr::generateName("subnet", 88, "declined-addresses"),
                         int64_t(0));
+                    isc::stats::StatsMgr::instance().setValue("declined-addresses", int64_t(0));
                 }
                 isc::stats::StatsMgr::instance().setValue(
                         isc::stats::StatsMgr::generateName("subnet", 44, "assigned-addresses"),
@@ -482,6 +487,7 @@ public:
                 isc::stats::StatsMgr::instance().setValue(
                         isc::stats::StatsMgr::generateName("subnet", 88, "assigned-addresses"),
                     int64_t(2));
+                isc::stats::StatsMgr::instance().setValue("assigned-addresses", int64_t(4));
             }
         }
     }
