@@ -27,6 +27,7 @@ namespace {
 using namespace isc::dhcp;
 using namespace isc::hooks;
 using namespace isc::limits;
+using namespace isc::stats;
 using namespace isc::util;
 using namespace std;
 
@@ -133,6 +134,16 @@ struct LeaseLimitFixture : ::testing::Test {
         handle.setArgument("fake_allocation", false);
         handle.setArgument(isc::util::formatDhcpSpace<D>("lease{}"), lease);
 
+        // Initialize stats to 0.
+        StatsMgr& stats_mgr = StatsMgr::instance();
+        if (D == DhcpSpace::DHCPv4) {
+            stats_mgr.setValue("pkt4-limit-exceeded", static_cast<int64_t>(0));
+            stats_mgr.setValue("pkt4-receive-drop", static_cast<int64_t>(0));
+        } else {
+            stats_mgr.setValue("pkt6-limit-exceeded", static_cast<int64_t>(0));
+            stats_mgr.setValue("pkt6-receive-drop", static_cast<int64_t>(0));
+        }
+
         // Simulate a leaseX_select callout call.
         EXPECT_NO_THROW_LOG(LimitManager::instance().lease_callout<D>(handle));
 
@@ -143,6 +154,29 @@ struct LeaseLimitFixture : ::testing::Test {
 
         // Expect the given status.
         EXPECT_EQ(handle.getStatus(), expected_status);
+
+        // Expect stats.
+        int64_t expected_stat = 0;
+        if (expected_status == CalloutHandle::NEXT_STEP_DROP) {
+            expected_stat = 1;
+        }
+        ObservationPtr limit_stat;
+        ObservationPtr drop_stat;
+        if (D == DhcpSpace::DHCPv4) {
+            limit_stat = stats_mgr.getObservation("pkt4-limit-exceeded");
+            ASSERT_TRUE(limit_stat);
+            EXPECT_EQ(expected_stat, limit_stat->getInteger().first);
+            drop_stat = stats_mgr.getObservation("pkt4-receive-drop");
+            ASSERT_TRUE(drop_stat);
+            EXPECT_EQ(expected_stat, drop_stat->getInteger().first);
+        } else {
+            limit_stat = stats_mgr.getObservation("pkt6-limit-exceeded");
+            ASSERT_TRUE(limit_stat);
+            EXPECT_EQ(expected_stat, limit_stat->getInteger().first);
+            drop_stat = stats_mgr.getObservation("pkt6-receive-drop");
+            ASSERT_TRUE(drop_stat);
+            EXPECT_EQ(expected_stat, drop_stat->getInteger().first);
+        }
     }
 
     /// @brief the body of most of the following tests
