@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2024 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -379,6 +379,8 @@ public:
     /// @param server_selector Server selector.
     /// @param code Option code.
     /// @param space Option space name.
+    /// @param client_classes Optional client classes list of the option to be deleted.
+    /// Defaults to an empty pointer.
     ///
     /// @return Pointer to the returned option or NULL if such option
     /// doesn't exist.
@@ -386,7 +388,8 @@ public:
                                   const Option::Universe& universe,
                                   const db::ServerSelector& server_selector,
                                   const uint16_t code,
-                                  const std::string& space);
+                                  const std::string& space,
+                                  const ClientClassesPtr client_classes = ClientClassesPtr());
 
     /// @brief Sends query to retrieve all global options.
     ///
@@ -628,10 +631,22 @@ public:
     /// Creates an Element tree of client class names and adds that to the end
     /// of the given bind array.
     ///
+    /// @param bindings PsqlBindArray to which the option value should be added.
     /// @param client_classes ClientClasses collection containing the class names
-    void addClientClassesBinding(db::PsqlBindArray& bindings, 
+    void addClientClassesBinding(db::PsqlBindArray& bindings,
                                  const ClientClasses& client_classes);
 
+    /// @brief Adds 'client-classes' to a bind array as a where clause argument.
+    ///
+    /// If client_classes parameter is not an empty pointer, it creates an Element
+    /// tree of client class names and adds that to the end of the given bind array.
+    /// Otherwise it adds a the LIKE operator wild-card string value, "%".
+    ///
+    /// @param bindings PsqlBindArray to which the option value should be added.
+    /// @param client_classes ClientClasses collection containing the class names.
+    void addClientClassesForWhereClause(db::PsqlBindArray& bindings,
+                                        const ClientClassesPtr client_classes = ClientClassesPtr());
+
     /// @brief Iterates over the class names in a JSON list element at a
     /// given column, adding each name to the given ClientClasses instance.
     ///
