@@ -12,15 +12,13 @@
 #include <dhcp_ddns/ncr_io.h>
 #include <dhcp_ddns/ncr_udp.h>
 #include <util/multi_threading_mgr.h>
-#include <test_utils.h>
+#include <util/ready_check.h>
 
 #include <boost/asio/ip/udp.hpp>
 
 #include <functional>
 #include <algorithm>
 
-#include <sys/select.h>
-
 using namespace std;
 using namespace isc;
 using namespace isc::asiolink;
