@@ -132,9 +132,11 @@ extern const isc::log::MessageID DHCPSRV_MEMFILE_LEASE_FILE_LOAD;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LEASE_LOAD;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LEASE_LOAD_ROW_ERROR;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_EXECUTE;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_FAIL_PID_CREATE;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_LEASE_FILE_RENAME_FAIL;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_LEASE_FILE_REOPEN_FAIL;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_RESCHEDULED;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_RUNNING;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_SETUP;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_SPAWN_FAIL;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_LFC_START;
