@@ -42,7 +42,7 @@ class LFCSetup;
 /// file will contain many entries for each lease. In order to mitigate this
 /// problem, the backend implements the Lease File Cleanup mechanism which is
 /// described on the Kea wiki:
-/// https://gitlab.isc.org/isc-projects/kea/wikis/designs/Lease-File-Cleanup-design.
+/// https://gitlab.isc.org/isc-projects/kea/-/wikis/Designs/lease-file-cleanup-design.
 ///
 /// The backend installs an @c asiolink::IntervalTimer to periodically execute
 /// the @c Memfile_LeaseMgr::lfcCallback. This callback function controls
@@ -963,6 +963,19 @@ public:
     /// support transactions, this is a no-op.
     virtual void rollback() override;
 
+    /// @brief Check if LFC is running.
+    ///
+    /// @return True if LFC is running, false otherwise (memfile only).
+    static bool isLFCProcessRunning(const std::string file_name, Universe u);
+
+    /// @brief Return status information.
+    ///
+    /// @return Null or a map with the lease file name to add to
+    /// status-get command output.
+    ///
+    /// E.g. { "csv-lease-file": "/var/lib/kea/kea-leases4.csv" }
+    virtual data::ElementPtr getStatus() const override;
+
     //@}
 
     /// @name Public type and method used to determine file names for LFC.
@@ -993,7 +1006,7 @@ public:
     /// - LFC PID File: ".pid"
     ///
     /// See
-    /// https://gitlab.isc.org/isc-projects/kea/wikis/designs/Lease-File-Cleanup-design
+    /// https://gitlab.isc.org/isc-projects/kea/-/wikis/Designs/lease-file-cleanup-design
     /// for details.
     ///
     /// @param file_name A base file name to which suffix is appended.
@@ -1012,8 +1025,8 @@ public:
     ///
     /// @param u Universe (V4 or V6).
     /// @param filename optional filename to use.
-    std::string getDefaultLeaseFilePath(Universe u,
-                                        const std::string filename = "") const;
+    static std::string getDefaultLeaseFilePath(Universe u,
+                                               const std::string filename = "");
 
     /// @brief Returns an absolute path to the lease file.
     ///
@@ -1081,7 +1094,7 @@ private:
     /// The <filename>.2, <filename>.1 and <filename>.completed are the
     /// products of the lease file cleanups (LFC).
     /// See:
-    /// https://gitlab.isc.org/isc-projects/kea/wikis/designs/Lease-File-Cleanup-design
+    /// https://gitlab.isc.org/isc-projects/kea/-/wikis/Designs/lease-file-cleanup-design
     /// for details.
     ///
     /// @note: When the server starts up or is reconfigured it will try to
@@ -1099,6 +1112,7 @@ private:
     /// @todo Consider implementing delaying the lease files loading when
     /// the LFC is in progress by the specified amount of time.
     ///
+    /// @param u Universe (V4 or V6).
     /// @param filename Name of the lease file.
     /// @param lease_file An object representing a lease file to which
     /// the server will store lease updates.
@@ -1114,7 +1128,7 @@ private:
     /// @throw DbOpenError when it is found that the LFC is in progress.
     template<typename LeaseObjectType, typename LeaseFileType,
              typename StorageType>
-    bool loadLeasesFromFiles(const std::string& filename,
+    bool loadLeasesFromFiles(Universe u, const std::string& filename,
                              boost::shared_ptr<LeaseFileType>& lease_file,
                              StorageType& storage);
 
@@ -1154,6 +1168,9 @@ public:
     /// @brief Returns the status code returned by the last executed
     /// LFC process.
     int getLFCExitStatus() const;
+
+    /// @brief Returns the last lfc process id.
+    int getLFCLastPid() const;
     //@}
 
     /// @brief Creates and runs the IPv4 lease stats query
@@ -1296,7 +1313,7 @@ private:
     /// the lease files.
     ///
     /// For the design and the terminology used in this description refer to
-    /// the https://gitlab.isc.org/isc-projects/kea/wikis/designs/Lease-File-Cleanup-design.
+    /// the https://gitlab.isc.org/isc-projects/kea/-/wikis/Designs/lease-file-cleanup-design.
     ///
     /// If the method finds that the %Lease File Copy exists it simply runs
     /// the @c kea-lfc application.
@@ -1546,6 +1563,11 @@ public:
     /// @return Always return 0 as this function is a noop for not SQL backends.
     virtual size_t upgradeExtendedInfo4(const LeasePageSize& page_size) override;
 
+    /// @brief Handler for kea-lfc-start command.
+    ///
+    /// @returns Status of command.
+    virtual isc::data::ConstElementPtr lfcStartHandler() override;
+
 protected:
 
     /// Extended information / Bulk Lease Query shared interface.
